package uah.regex.gp.stats;

import java.io.File;
import java.io.IOException;

import ec.*;

//import ec.gp.GPIndividual;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.gp.GPSpecies;
import ec.gp.koza.KozaFitness;
import ec.util.*;

@SuppressWarnings("serial")
public class GPRegexStatistics extends Statistics {
	/** compress? */
	public static final String P_COMPRESS = "gzip";

	public Individual[] best_of_run_a;
	public long totalNodes[];
	public long totalDepths[];

	/** log file parameter */
	public static final String P_STATISTICS_FILE = "file";

	/** The Statistics' log */
	public int statisticslog;

	/* The best individual we've found so far */
	//public Individual best_of_run;

	public GPRegexStatistics() { /*best_of_run = null;*/
		statisticslog = 0; /* stdout */
	}

	public void setup(final EvolutionState state, final Parameter base) {
		super.setup(state, base);
		File statisticsFile = state.parameters.getFile(base
				.push(P_STATISTICS_FILE), null);

		if (statisticsFile != null)
			try {
				statisticslog = state.output.addLog(statisticsFile,
						Output.V_NO_GENERAL - 1, false,
						!state.parameters.getBoolean(base.push(P_COMPRESS),
								null, false), state.parameters.getBoolean(base
								.push(P_COMPRESS), null, false));
			} catch (IOException i) {
				state.output
						.fatal("An IOException occurred while trying to create the log "
								+ statisticsFile + ":\n" + i);
			}
	}

	public void preInitializationStatistics(final EvolutionState state) {
		super.preInitializationStatistics(state);

	}

	public void postInitializationStatistics(final EvolutionState state) {
		super.postInitializationStatistics(state);
		// set up our best_of_run array -- can't do this in setup, because
		// we don't know if the number of subpopulations has been determined yet
		best_of_run_a = new Individual[state.population.subpops.length];

		// print out our generation number
		state.output.print("0 ", Output.V_NO_GENERAL, statisticslog);

		totalNodes = new long[state.population.subpops.length];
		for (int x = 0; x < totalNodes.length; x++)
			totalNodes[x] = 0;
		totalDepths = new long[state.population.subpops.length];
		for (int x = 0; x < totalDepths.length; x++)
			totalDepths[x] = 0;
	}

	public void preBreedingStatistics(final EvolutionState state) {
		super.preBreedingStatistics(state);

	}

	public void postBreedingStatistics(final EvolutionState state) {
		super.postBreedingStatistics(state);
		state.output.print("" + (state.generation + 1) + " ",
				Output.V_NO_GENERAL, statisticslog); // 1 because we're putting the breeding info on the same line as the generation it *produces*, and the generation number is increased *after* breeding occurs, and statistics for it
	}

	public void preEvaluationStatistics(final EvolutionState state) {
		super.preEvaluationStatistics(state);
	}

	/** Prints out the statistics, but does not end with a println --
	    this lets overriding methods print additional statistics on the same line */
	protected void _postEvaluationStatistics(final EvolutionState state) {

		Individual[] best_i = new Individual[state.population.subpops.length]; // quiets compiler complaints

		for (int x = 0; x < state.population.subpops.length; x++) {

			long totNodesPerGen = 0;
			long totDepthPerGen = 0;

			// check to make sure they're the right class
			if (!(state.population.subpops[x].species instanceof GPSpecies))
				state.output.fatal("Subpopulation "
					+ x
					+ " is not of the species form GPSpecies."
					+ "  Cannot do timing statistics with KozaShortStatistics.");

			long[] numNodes = new long[((GPIndividual) (state.population.subpops[x].species.i_prototype)).trees.length];
			long[] numDepth = new long[((GPIndividual) (state.population.subpops[x].species.i_prototype)).trees.length];

			for (int y = 0; y < state.population.subpops[x].individuals.length; y++) {
				GPIndividual i = (GPIndividual) (state.population.subpops[x].individuals[y]);
				for (int z = 0; z < i.trees.length; z++) {
					numNodes[z] += i.trees[z].child
							.numNodes(GPNode.NODESEARCH_ALL);
					numDepth[z] += i.trees[z].child.depth();
				}
			}

			for (int tr = 0; tr < numNodes.length; tr++)
				totNodesPerGen += numNodes[tr];

			totalNodes[x] += totNodesPerGen;
            state.output.print(" " + ((double)totNodesPerGen)/state.population.subpops[x].individuals.length, Output.V_NO_GENERAL, statisticslog);
			state.output.print(" " + ((double) totalNodes[x])
						/ (state.population.subpops[x].individuals.length * (state.generation + 1))
						+ " ", Output.V_NO_GENERAL, statisticslog);

			for (int tr = 0; tr < numDepth.length; tr++)
				totDepthPerGen += numDepth[tr];

			totalDepths[x] += totDepthPerGen;
            state.output.print(" " + ((double)totDepthPerGen)/
                    (state.population.subpops[x].individuals.length *
                     numDepth.length), Output.V_NO_GENERAL, statisticslog);
            
			state.output.print(" " + ((double) totalDepths[x])
						/ (state.population.subpops[x].individuals.length * (state.generation + 1))
						+ " ", Output.V_NO_GENERAL, statisticslog);

			// fitness information
			float meanRaw = 0.0f;
			float meanAdjusted = 0.0f;
			long hits = 0;

			if (!(state.population.subpops[x].species.f_prototype instanceof KozaFitness))
				state.output.fatal("Subpopulation " + x
								+ " is not of the fitness KozaFitness.  Cannot do timing statistics with KozaStatistics.");

			best_i[x] = null;
			for (int y = 0; y < state.population.subpops[x].individuals.length; y++) {
				// best individual
				if (best_i[x] == null
						|| state.population.subpops[x].individuals[y].fitness
								.betterThan(best_i[x].fitness))
					best_i[x] = state.population.subpops[x].individuals[y];

				// mean for population
				meanRaw += ((KozaFitness) (state.population.subpops[x].individuals[y].fitness))
						.rawFitness();
				meanAdjusted += ((KozaFitness) (state.population.subpops[x].individuals[y].fitness))
						.adjustedFitness();
				hits += ((KozaFitness) (state.population.subpops[x].individuals[y].fitness)).hits;
			}

			// compute fitness stats
			meanRaw /= state.population.subpops[x].individuals.length;
			meanAdjusted /= state.population.subpops[x].individuals.length;
			state.output.print("" + meanRaw + " " + meanAdjusted + " "
					+ ((double) hits)
					/ state.population.subpops[x].individuals.length + " ",
					Output.V_NO_GENERAL, statisticslog);
			state.output.print(""
					+ ((KozaFitness) (best_i[x].fitness)).rawFitness() + " "
					+ ((KozaFitness) (best_i[x].fitness)).adjustedFitness()
					+ " " + ((KozaFitness) (best_i[x].fitness)).hits + " ",
					Output.V_NO_GENERAL, statisticslog);

			// now test to see if it's the new best_of_run_a[x]
			if (best_of_run_a[x] == null
					|| best_i[x].fitness.betterThan(best_of_run_a[x].fitness))
				best_of_run_a[x] = best_i[x];

			state.output.print(""
					+ ((KozaFitness) (best_of_run_a[x].fitness)).rawFitness()
					+ " "
					+ ((KozaFitness) (best_of_run_a[x].fitness))
							.adjustedFitness() + " "
					+ ((KozaFitness) (best_of_run_a[x].fitness)).hits + " ",
					Output.V_NO_GENERAL, statisticslog);
		}
	}

	public void postEvaluationStatistics(final EvolutionState state) {
		super.postEvaluationStatistics(state);
		_postEvaluationStatistics(state);
		state.output.println("", Output.V_NO_GENERAL, statisticslog);
	}
}
